/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : CPrimitive.cpp

    Nyelv             : C++

    Ltrehozs dtuma : 2008.08.28

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.4. Sznes geometria megjelentse
						305. oldal

               Copyright (C) Nyisztor Kroly 2008-2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Szksges fggvnyknyvtrak --------------------------------------------*/
// biztostsuk a szksges fggvnyknyvtrak linkelst!

/*- Befordtott fejllomnyok -----------------------------------------------*/
#include "CPrimitive.h"

#include "..\..\common\util\util.h"
using namespace app::util;

#include "..\..\common\util\core.h"
using app::util::Core;


/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
/*!
 * \brief
 * Write brief comment for scene here.
 * 
 * Write detailed description for scene here.
 * 
 * \remarks
 * Write remarks for scene here.
 * 
 * \see
 * Separate items with the '|' character.
 */
namespace scene
{

/*- Defincik s konstansok ------------------------------------------------*/
/*- Globlis vltozk -------------------------------------------------------*/

/*- Implementcis rsz -----------------------------------------------------*/

CPrimitive::CPrimitive( const string& fxFileName_in,
						const D3DXVECTOR3& posVec_in, 
						const D3DXVECTOR3& rotVec_in, 
						const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFileName_in )
{
	myFaultFlag = true;
	// D3DDevice mutat lekrdezse
	myD3DDevicePtr = GameCore->Device();
	if( !myD3DDevicePtr )
	{
		const wchar_t errorStr[] = L"Nincs rvnyes Direct3D eszkz mutat!";
		::MessageBoxW(0, errorStr, 0, 0);
		return;
	}

	D3DXMatrixIdentity( &myWorldMat );
	D3DXMATRIX mat;
	D3DXMatrixScaling( &mat, scaleVec_in.x, scaleVec_in.y, scaleVec_in.z );
	myWorldMat *= mat;
	D3DXMatrixRotationYawPitchRoll( &mat, rotVec_in.y, rotVec_in.x, rotVec_in.z );
	myWorldMat *= mat;
	D3DXMatrixTranslation( &mat, posVec_in.x, posVec_in.y, posVec_in.z );
	myWorldMat *= mat;

	myTriangleListVBPtr = NULL;
	myEffectPtr = NULL;
	// vertex adatformtum definilsa
	if( !initVertexDeclaration() )
	{
		const wchar_t errorStr[] = L"Sikertelen vertex deklarls!";
		::MessageBoxW(0, errorStr, 0, 0);
		return;
	}
	// vertex puffer feltltse 
	if( !initVertexBuffers() )
	{
		const wchar_t errorStr[] = L"Sikertelen vertex puffer ltrehozs s feltlts!";
		::MessageBoxW(0, errorStr, 0, 0);
		return;
	}
	// effekt betltse
	if( !loadFX() )
	{
		wchar_t errorStr[MAX_PATH];
		swprintf_s( errorStr, L"Sikertelen effect-betlts: %s", fxFileName_in );
		::MessageBoxW(0, errorStr, 0, 0);
		return;
	}

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betlts s inicializls. Fjl: %s", fxFileName_in.c_str() );
	LOG_INFO( messStr )

	myFaultFlag = false;
}

CPrimitive::~CPrimitive()
{
	SAFE_RELEASE( myTriangleListVBPtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
	
}

/*!
 * \brief Vertex-struktra megadsa
 * \remarks a shader ez alapjn kezeli a cscspontokat
 */
bool CPrimitive::initVertexDeclaration()
{
	//3D-koordinta + cscspont szn rtkekbl ll vertex struktra
	D3DVERTEXELEMENT9 vertexElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		{0, 12,  D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR, 0},
		D3DDECL_END()
	};
	if( FAILED( myD3DDevicePtr->CreateVertexDeclaration( vertexElements, &myDeclPtr ) ) )
	{
		LOG_ERROR( "Sikertelen Vertex deklarls" );
		return false;
	}

	LOG_INFO( "Vertex deklarls rendben" );
	return true;
}

bool CPrimitive::initVertexBuffers()
{
	// ez a pointer a vertexpuffer adatterletre fog mutatni
	SVertexPosCol* vertPtr = NULL;
	// 1) hromszg lista
	SVertexPosCol triangleList[] =
	{
		D3DXVECTOR3( -0.5f, 0.0f, 1.0f ), D3DCOLOR_XRGB( 255,   0,   0 )/*D3DXCOLOR(1.f, 0.0f, 0.f, 1.f)*/,// hromszg cscspontjai
		D3DXVECTOR3(  0.0f, 0.5f, 1.0f ), D3DCOLOR_XRGB(   0, 255,   0 ),
		D3DXVECTOR3(  0.5f, 0.0f, 1.0f ), D3DCOLOR_XRGB(   0,   0, 255 )
	};

	if( FAILED( myD3DDevicePtr->CreateVertexBuffer( 3 * sizeof( SVertexPosCol ),
													0, 
													0,
													D3DPOOL_DEFAULT, 
													&myTriangleListVBPtr,
													NULL ) ) )
	{
		return false;
	}
	// megszerezzk majd zroljuk a vertex puffer memriaterlett, hogy tmsolhassuk a vertex adatokat
	if( FAILED( myTriangleListVBPtr->Lock( 0, sizeof( triangleList ), (VOID**)&vertPtr, 0 ) ) )
	{
		return false;
	}

	// vertex adatok msolsa
	memcpy( vertPtr, triangleList, sizeof( triangleList ) );

	// vertex puffer zrolsnak feloldsa
	myTriangleListVBPtr->Unlock();

	return true;
}

bool CPrimitive::loadFX()
{
	// ltezik a fjl? - megprbljuk megnyitni
	CHECK_WEXIT( fileExists(myFXFile) )
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	
	D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0/*D3DXSHADER_DEBUG*/, 
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							);
	// effekt betltsi hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		return false;
	}

	// kezelk lekrdezse
	myWireframeHandle  = myEffectPtr->GetTechniqueByName("WireframeColorTech");
	myFilledTechHandle = myEffectPtr->GetTechniqueByName("FilledColorTech");
	myHWVP        = myEffectPtr->GetParameterByName(0, "gWorldViewProjection");

	return true;
}


void CPrimitive::render()
{	
	myD3DDevicePtr->SetStreamSource( 0, myTriangleListVBPtr, 0, sizeof(SVertexPosCol) );
	myD3DDevicePtr->SetVertexDeclaration(myDeclPtr);

	myEffectPtr->SetMatrix(myHWVP, &( myWorldMat * GameCore->Camera()->getViewProjMatrix() ));
	// konstans rnyals
	myD3DDevicePtr->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_FLAT );
	// Gouraud rnyals
	myD3DDevicePtr->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_GOURAUD );

	myEffectPtr->SetTechnique( myWireframeHandle );
	{
		UINT passCnt = 0;

		myEffectPtr->Begin( &passCnt, 0 );
		for( UINT p = 0; p < passCnt; ++p )
		{
			myEffectPtr->BeginPass( p );
	        
			myD3DDevicePtr->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 1 );

			myEffectPtr->EndPass();
		}
		myEffectPtr->End();
	}
	D3DXMATRIX mat;
	// 1 egysgnyi eltols pozitv irnyba (jobbra) az x-tengely mentn
	D3DXMatrixTranslation( &mat, 1.f, 0, 0 );
	
	myEffectPtr->SetMatrix(myHWVP, &( (myWorldMat * mat) * GameCore->Camera()->getViewProjMatrix() ));

	myEffectPtr->SetTechnique( myFilledTechHandle );
	{
		UINT passCnt = 0;

		myEffectPtr->Begin( &passCnt, 0 );
		for( UINT p = 0; p < passCnt; ++p )
		{
			myEffectPtr->BeginPass( p );
	        
			myD3DDevicePtr->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 1 );

			myEffectPtr->EndPass();
		}
		myEffectPtr->End();
	}
}

void CPrimitive::onLostDevice()
{
	myEffectPtr->OnLostDevice();
	SAFE_RELEASE( myTriangleListVBPtr );
}

void CPrimitive::onResetDevice()
{
	initVertexBuffers();
	myEffectPtr->OnResetDevice();
}

}
}

