/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : CAmbientLitMeshNode.cpp

    Nyelv             : C++

    Ltrehozs dtuma : 2008.12.26

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.6.1. "ltalnos httrmegvilgts"
						323. oldal

               Copyright (C) Nyisztor Kroly 2008-2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Szksges fggvnyknyvtrak --------------------------------------------*/
// biztostsuk a szksges fggvnyknyvtrak linkelst!

/*- Befordtott fejllomnyok -----------------------------------------------*/
#include "CAmbientLitMeshNode.h"

#include "..\..\common\scene\BaseMesh.h"

#include "..\..\common\util\core.h"
using app::util::Core;

#include "..\..\common\util\util.h"
using namespace app::util;
using namespace app::debug;

/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
namespace scene
{

/*- Defincik s konstansok ------------------------------------------------*/

/*- Globlis vltozk -------------------------------------------------------*/

/*- Implementcis rsz -----------------------------------------------------*/

CAmbientLitMeshNode::CAmbientLitMeshNode( const string& fxFileName_in, 
				  const string& meshFile_in,
				  const D3DXVECTOR3& posVec_in, 
				  const D3DXVECTOR3& rotVec_in, 
				  const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFileName_in ), 
													 myPosVec(posVec_in), 
													 myRotVec(rotVec_in), 
													 myScaleVec(scaleVec_in)
{
	myMeshPtr = new BaseMesh( meshFile_in );

	// D3DDevice mutat lekrdezse
	myD3DDevicePtr = GameCore->Device();
	if( !myD3DDevicePtr )
	{
		const wchar_t errorStr[] = L"Nincs rvnyes Direct3D eszkz mutat!";
		::MessageBoxW(0, errorStr, 0, 0);
		return;
	}

	D3DXMatrixIdentity( &myWorldMat );
	
	D3DXMatrixScaling( &myScaleMat, myScaleVec.x, myScaleVec.y, myScaleVec.z );
	
	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	
	D3DXMatrixTranslation( &myTranslMat, myPosVec.x, myPosVec.y, myPosVec.z );
	
	myAmbientMat = D3DXCOLOR( 0.5f, 0.5f, 1.0f, 1.0f );
	myAmbientLight = D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f );

	myEffectPtr = NULL;
	
	CHECK_WEXIT( initVertexDeclaration() )

	CHECK_WEXIT( loadFX() )

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betlts s inicializls. Fjl: %s", fxFileName_in.c_str() );
	LOG_INFO( messStr )
}

CAmbientLitMeshNode::~CAmbientLitMeshNode()
{
	SAFE_RELEASE( myMeshPtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
}

/*!
 * \brief Vertex-struktra megadsa
 * \remarks a shader ez alapjn kezeli a cscspontokat
 */
HRESULT CAmbientLitMeshNode::initVertexDeclaration()
{
	//3D helyvektort definil struktra
	D3DVERTEXELEMENT9 vertexPosElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		D3DDECL_END()
	};
	CHECK_WEXIT( myD3DDevicePtr->CreateVertexDeclaration( vertexPosElements, &myDeclPtr) );

	LOG_INFO( "Vertex deklarls rendben" );
	CHECK_WEXIT( myMeshPtr->setVertexDecl( vertexPosElements ) );

	return S_OK;
}


HRESULT CAmbientLitMeshNode::loadFX()
{
	// ltezik a fjl? - megprbljuk megnyitni
	CHECK_WEXIT( fileExists(myFXFile) )
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	
	CHECK_WEXIT( D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0/*/D3DXSHADER_DEBUG*/, 
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							) );
	// effekt betltsi hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		return E_FAIL;
	}

	// kezelk lekrdezse
	mhTech       = myEffectPtr->GetTechniqueByName("AmbientTech");
	myHWVP       = myEffectPtr->GetParameterByName(0, "gWorldViewProjection");
	myHAmbientMaterialColor	= myEffectPtr->GetParameterByName(0, "gAmbientMaterialColor");
	myHAmbientLightColor    = myEffectPtr->GetParameterByName(0, "gAmbientLightColor");

	if( mhTech == NULL
		|| myHWVP == NULL
		|| myHAmbientMaterialColor == NULL
		|| myHAmbientLightColor == NULL )
	{
		LOG_ERROR( "Effekt paramter beolvassi hiba!" );
		return E_FAIL;
	}

	//HR(myEffectPtr->SetMatrix(mhWorldInv, &mSceneWorldInv));
	myEffectPtr->SetTechnique(mhTech);

	return S_OK;
}

const D3DXMATRIX& CAmbientLitMeshNode::getWorldMat() 
{		
		myWorldMat = myScaleMat * myRotMat * myTranslMat;
		return myWorldMat;
}

void CAmbientLitMeshNode::update( float dt )
{
	// lassan forgatjuk a modellt a sajt tengelye krl
	static const D3DXVECTOR3 rotationStep( 0, 0.1f, 0 );
	myRotVec += rotationStep * dt;

	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
}

void CAmbientLitMeshNode::render()
{
	//myEffectPtr->SetValue(mhLight, &mLight, sizeof(DirLight));
	myEffectPtr->SetMatrix(myHWVP, &( getWorldMat() * GameCore->Camera()->getViewProjMatrix() ));
	
	myEffectPtr->SetValue(myHAmbientMaterialColor,	&myAmbientMat,		sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHAmbientLightColor,		&myAmbientLight,	sizeof(D3DXCOLOR));
	
	myMeshPtr->render( myEffectPtr, NULL /* no tex */, NULL /* no mat */, true /* draw opaque subsets */ );
}

void CAmbientLitMeshNode::onLostDevice()
{
	myEffectPtr->OnLostDevice();
}

void CAmbientLitMeshNode::onResetDevice()
{
	myEffectPtr->OnResetDevice();
}

}
}

