/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: Pointlight.fx
	Lers	: Pontszer fny
	Lsd	: 11.6.4. "Pontszer fnyforrsok"
			  342. oldal
------------------------------------------------- */

uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldViewProjection;
uniform extern float4 gAmbientMaterialColor;
uniform extern float4 gDiffuseMaterialColor;
uniform extern float4 gSpecularMaterialColor;
uniform extern float4 gLightColor;
uniform extern float3 gLightPos;
uniform extern float3 gCameraPos;

uniform extern float gSpecularParam;
// egytthat (enyhn mdostott Lambert-fle tvolsgfggvnyhez) 
uniform extern float gFactor;
 
struct VS_OUTPUT
{
    float4 PositionProj : POSITION0;
    float4 Color	: COLOR0;
};
// cscspont rnyal
VS_OUTPUT vs_main(float3 pos_in : POSITION0, float3 normal_in : NORMAL0)
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;
	
	// felleti normlvektor transzformlsa modelltrbl a vilg-koordintarendszerbe
	float3 normalW = mul(float4(normal_in, 0.0f), gWorld).xyz;
	normalW = normalize(normalW);
	
	// cscspont helyzetnek transzformcija vilgkoordinta-rendszerbe
	float3 posW = mul(float4(pos_in, 1.0f), gWorld);
	// fnyvektor kiszmtsa a fnyforrs s a vertex helyzete alapjn
	float3 lightVecW = normalize(gLightPos - posW);
	
	// objektum-fnyforrs tvolsg mrtke
	float d = distance(gLightPos, posW);
	
	// Lambert-fle tvolsg fggvny
	float attenuation = gFactor * d * d;
	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i_diffuse = max(dot(lightVecW, normalW), 0.0f);
	// a diffz szn kiszmtsa
	float3 diffuseColor = i_diffuse*( gDiffuseMaterialColor * gLightColor).rgb;
	
	// objektum-kamera vektor kiszmtsa
	float3 camObjDirW = normalize(gCameraPos - posW);
	// r (felletrl visszavert fny) kiszmtsa
	float3 r = reflect(-lightVecW, normalW);
	// a fnyfolt intenzitsnak kiszmtsa
	float i_spec = pow(max(dot(r,camObjDirW), 0.0f), gSpecularParam);
	// fnyfolt sznnek kiszmtsa
	float3 specColor = i_spec*( gSpecularMaterialColor * gLightColor).rgb;
	
	// kimeneti struktra feltltse
	// biztostsuk, hogy a sznkomponensek a [0,1] lebegpontos intervallumon bell maradnak
	outVS.Color.rgb = saturate(gAmbientMaterialColor + (diffuseColor + specColor)/attenuation);
	outVS.Color.a   = gDiffuseMaterialColor.a;
	
	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(pos_in, 1.0f), gWorldViewProjection);
	
	// a kitlttt struktra visszatrtse
    return outVS;
}
// kppont rnyal
float4 ps_main(float4 Color : COLOR0) : COLOR
{
    return Color;
}

technique PointLightTech
{
    pass P0
    {
        // a P0 lps cscspont- s kppontrnyalja
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}

