/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: SpecularPhong.fx
	Lers	: Perpixel fnyfoltkpz  megvilgtsi modell
	Lsd	: 11.6.6. Kppontonknti megvilgts  a Phong-rnyals
			  351. oldal
------------------------------------------------- */

uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldViewProjection;
uniform extern float4 gAmbientMaterialColor;
uniform extern float4 gDiffuseMaterialColor;
uniform extern float4 gSpecularMaterialColor;
uniform extern float4 gLightColor;
uniform extern float3 gLightPos;
uniform extern float3 gCameraPos;

uniform extern float gSpecularParam;
 
struct VS_OUTPUT
{
    float4 PositionProj : POSITION0;
	float3 NormalW 		: TEXCOORD0;
    float3 CamObjDirW 	: TEXCOORD1;
	float3 LightVecW 	: TEXCOORD2;
};
// cscspont rnyal
VS_OUTPUT vs_main(float3 posLocal_in : POSITION0, float3 normalLocal_in : NORMAL0)
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;
	
	// felleti normlvektor transzformlsa modelltrbl a vilg-koordintarendszerbe
	float3 normalW = mul(float4(normalLocal_in, 0.0f), gWorld).xyz;
	outVS.NormalW = normalize(normalW);

	// cscspont helyzetnek transzformcija vilgkoordinta-rendszerbe
	float3 posW = mul(float4(posLocal_in, 1.0f), gWorld).xyz;
	// objektum-kamera vektor kiszmtsa
	outVS.CamObjDirW = normalize(gCameraPos - posW);
	// fnyvektor kiszmtsa
	outVS.LightVecW = normalize( gLightPos - posW );
	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(posLocal_in, 1.0f), gWorldViewProjection);	
	// a kitlttt struktra visszatrtse
    return outVS;
}

// kppont rnyal
float4 ps_main(float3 normalW_in : TEXCOORD0, float3 camObjDirW_in : TEXCOORD1, float3 lightVecW_in : TEXCOORD2) : COLOR
{
	// normalizljuk a raszterizcis szakaszban 
	// kiszmtott normlvektort, ugyanis 
	// az interpolci nem eredmnyez felttlenl 
	// egysgvektorokat
	float3 normalWorld = normalize(normalW_in);
	float3 viewWorld = normalize(camObjDirW_in);
	float3 lightWorld = normalize(lightVecW_in);
	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i = max(dot(lightWorld, normalWorld), 0.0f);
	// a diffz szn kiszmtsa
	float3 diffuseColor = i*( gDiffuseMaterialColor * gLightColor).rgb;
	// a felletrl visszavert fny irnynak kiszmtsa
	float3 r = reflect(-lightWorld, normalWorld);
	// a fnyfolt intenzitsnak kiszmtsa
	float i_spec = pow(max(dot(r,viewWorld),0.0f), gSpecularParam);
	// fnyfolt sznnek kiszmtsa
	float3 specColor = i_spec*( gSpecularMaterialColor * gLightColor).rgb;	
	// kimeneti struktra feltltse
	return float4( gAmbientMaterialColor + diffuseColor + specColor, gDiffuseMaterialColor.a );    
}

technique AmbientDiffuseSpecularTech
{
    pass P0
    {
        // a P0 lps cscspont- s kppontrnyalja
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}
