/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : CSpecularLitMeshNode.h

    Nyelv             : C++

    Ltrehozs dtuma : 2008.07.08

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.6.6. Kppontonknti megvilgts  a Phong-rnyals
						351. oldal

               Copyright (C) Nyisztor Kroly 2008-2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Szksges fggvnyknyvtrak --------------------------------------------*/
// biztostsuk a szksges fggvnyknyvtrak linkelst!

/*- Befordtott fejllomnyok -----------------------------------------------*/
#include "CSpecularLitMeshNode.h"

#include "..\..\common\scene\BaseMesh.h"

#include "..\..\common\util\core.h"
using app::util::Core;

#include "..\..\common\util\util.h"
using namespace app::util;
using namespace app::debug;

/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
namespace scene
{

/*- Defincik s konstansok ------------------------------------------------*/
// statikus adattagok inicializlsa
/*- Globlis vltozk -------------------------------------------------------*/

/*- Implementcis rsz -----------------------------------------------------*/

CSpecularLitMeshNode::CSpecularLitMeshNode( const string& fxFileName_in, 
				  const string& meshFile_in,
				  const D3DXVECTOR3& posVec_in, 
				  const D3DXVECTOR3& rotVec_in, 
				  const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFileName_in ), 
													 myPosVec(posVec_in), 
													 myRotVec(rotVec_in), 
													 myScaleVec(scaleVec_in)
{
	myDeclPtr = NULL;

	myMeshPtr = new BaseMesh( meshFile_in );
	// D3DDevice mutat lekrdezse
	myD3DDevicePtr = GameCore->Device();

	D3DXMatrixIdentity( &myWorldMat );
	
	D3DXMatrixScaling( &myScaleMat, myScaleVec.x, myScaleVec.y, myScaleVec.z );
	
	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	
	D3DXMatrixTranslation( &myTranslMat, myPosVec.x, myPosVec.y, myPosVec.z );
	
	myAmbientMat = D3DCOLOR_XRGB( 16, 16, 16 );
	myDiffuseMat = D3DCOLOR_XRGB( 64, 64, 64 );
	mySpecularMat = D3DCOLOR_XRGB( 255, 255, 255 );
	myDiffuseLight = D3DCOLOR_XRGB( 255, 255, 255 );
	
	myLightPos = D3DXVECTOR3( -10.f, 10.f, -50.f );		
	// a fnyfoltkpz anyagtulajdonsg
	// magasabb rtkek lesebb fnyfoltokat eredmnyeznek
	mySpecularPower = 16.0f;

	myEffectPtr = NULL;
	
	CHECK_WEXIT( initVertexDeclaration() )

	CHECK_WEXIT( loadFX() )

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betlts s inicializls. Fjl: %s", fxFileName_in.c_str() );
	LOG_INFO( messStr )
}

CSpecularLitMeshNode::~CSpecularLitMeshNode()
{
	SAFE_RELEASE( myMeshPtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
}

/*!
 * \brief Vertex-struktra megadsa
 * \remarks a shader ez alapjn kezeli a cscspontokat
 */
HRESULT CSpecularLitMeshNode::initVertexDeclaration()
{
	// vertex deklarci
	D3DVERTEXELEMENT9 vertexPosNormalElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		{0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0},
		D3DDECL_END()
	};
	if( FAILED( myD3DDevicePtr->CreateVertexDeclaration( vertexPosNormalElements, &myDeclPtr)) )
	{
		LOG_ERROR( "Sikertelen vertex deklarls" );
		return E_FAIL;
	}

	CHECK_WEXIT( myMeshPtr->setVertexDecl( vertexPosNormalElements ) );
	LOG_INFO( "Vertex deklarls rendben" );
	return S_OK;
}


HRESULT CSpecularLitMeshNode::loadFX()
{
	// ltezik a fjl? - megprbljuk megnyitni
	CHECK_WEXIT( fileExists(myFXFile) )
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	
	CHECK_WLOG( D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0/*D3DXSHADER_DEBUG*/, 
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							) );
	// effekt betltsi hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		return E_FAIL;
	}

	// kezelk lekrdezse
	myHTech       = myEffectPtr->GetTechniqueByName("AmbientDiffuseSpecularTech");
	myHWVP        = myEffectPtr->GetParameterByName(0, "gWorldViewProjection");
	myHAmbientMaterialColor	= myEffectPtr->GetParameterByName(0, "gAmbientMaterialColor");
	myHDiffuseMaterialColor = myEffectPtr->GetParameterByName(0, "gDiffuseMaterialColor");
	myHSpecularMaterialColor = myEffectPtr->GetParameterByName(0, "gSpecularMaterialColor");
	myHSpecularPower = myEffectPtr->GetParameterByName(0, "gSpecularParam" );
	
	myHWorld 	= myEffectPtr->GetParameterByName(0, "gWorld");
	myHLightPos = myEffectPtr->GetParameterByName(0, "gLightPos");
	myHCameraPos = myEffectPtr->GetParameterByName(0, "gCameraPos");
	
	myHLightColor = myEffectPtr->GetParameterByName(0, "gLightColor");	

	if( !myHTech
		|| !myHWVP
		|| !myHAmbientMaterialColor
		|| !myHDiffuseMaterialColor
		|| !myHSpecularMaterialColor
		|| !myHSpecularPower
		|| !myHWorld
		|| !myHLightPos
		|| !myHCameraPos
		|| !myHLightColor	)
	{
		LOG_ERROR( "Effekt paramter beolvassi hiba!" );
		return E_FAIL;
	}

	CHECK_WEXIT( myEffectPtr->SetFloat( myHSpecularPower, mySpecularPower ) );

	return S_OK;
}

const D3DXMATRIX& CSpecularLitMeshNode::getWorldMat() 
{		
		myWorldMat = myScaleMat * myRotMat * myTranslMat;
		return myWorldMat;
}

void CSpecularLitMeshNode::update( float dt )
{
	// lassan forgatjuk a modellt a sajt tengelye krl
	static const D3DXVECTOR3 rotationStep( 0.f, 0.6f, 0 );
	myRotVec += rotationStep * dt;

	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
}

void CSpecularLitMeshNode::render()
{
	CHECK_WEXIT( myEffectPtr->SetTechnique(myHTech) );
	myD3DDevicePtr->SetVertexDeclaration(myDeclPtr);
	
	myEffectPtr->SetMatrix(myHWVP, &( getWorldMat() * GameCore->Camera()->getViewProjMatrix() ));
		
	myEffectPtr->SetValue(myHAmbientMaterialColor, &myAmbientMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHDiffuseMaterialColor, &myDiffuseMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHSpecularMaterialColor, &mySpecularMat, sizeof(D3DXCOLOR));
	
	myEffectPtr->SetValue(myHLightPos, &myLightPos, sizeof(D3DXVECTOR3));
	myEffectPtr->SetValue(myHLightColor, &myDiffuseLight, sizeof(D3DXCOLOR));
		
	myEffectPtr->SetMatrix(myHWorld, &getWorldMat());

	myEffectPtr->SetValue( myHCameraPos, &( GameCore->Camera()->getPosition() ), sizeof(D3DXVECTOR3) );
	
	
	myMeshPtr->render( myEffectPtr, NULL /* no tex */, NULL /* no mat */, true /* draw opaque subsets */ );
}

void CSpecularLitMeshNode::onLostDevice()
{
	myEffectPtr->OnLostDevice();
}

void CSpecularLitMeshNode::onResetDevice()
{
	myEffectPtr->OnResetDevice();
}

}
}

