/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: TexturedAnim.fx
	Lers	: textra nimci
	Lsd	: 11.7.2. Textrakoordinta-alap animci
			  373. oldal
------------------------------------------------- */
uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldViewProjection;
uniform extern float4 gAmbientMaterialColor;
uniform extern float4 gDiffuseMaterialColor;
uniform extern float4 gSpecularMaterialColor;
uniform extern float4 gLightColor;
uniform extern float3 gLightPos;
uniform extern float3 gCameraPos;
uniform extern float gSpecularParam;

uniform extern float gTime;

const float2 Tex0Step = float2(0.5f, 0.25f);
uniform extern texture gTex0;
sampler SamplerTex0 = sampler_state
{
	Texture = <gTex0>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

struct VS_OUTPUT
{
    float4 PositionProj : POSITION0;
	float2 Tex0			: TEXCOORD0;
	float3 NormalW 		: TEXCOORD1;
    float3 CamObjDirW 	: TEXCOORD2;
	float3 LightVecW 	: TEXCOORD3;
};
// cscspont rnyal
VS_OUTPUT vs_main(float3 posLocal_in : POSITION0, 
				  float3 normalLocal_in : NORMAL0, 
				  float2 tex0_in : TEXCOORD0)
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;
	
	float2 tex0Coord = Tex0Step * gTime;
	
	if(tex0Coord.x >= 1.f)
		tex0Coord.x  -= 1.f;
	if(tex0Coord.y >= 1.f)
		tex0Coord.y  -= 1.f;
	
	 outVS.Tex0 = tex0_in + tex0Coord;
	// felleti normlvektor transzformlsa modelltrbl a vilg-koordintarendszerbe
	float3 normalW = mul(float4(normalLocal_in, 0.0f), gWorld).xyz;
	outVS.NormalW = normalize(normalW);

	// cscspont helyzetnek transzformcija vilgkoordinta-rendszerbe
	float3 posW = mul(float4(posLocal_in, 1.0f), gWorld).xyz;
	// objektum-kamera vektor kiszmtsa
	outVS.CamObjDirW = normalize(gCameraPos - posW);
	// fnyvektor kiszmtsa
	outVS.LightVecW = normalize( gLightPos - posW );
	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(posLocal_in, 1.0f), gWorldViewProjection);	
	// a kitlttt struktra visszatrtse
    return outVS;
}

// kppont rnyal
float4 ps_main(float2 tex0_in : TEXCOORD0, 
			   float3 normalW_in : TEXCOORD1, 
			   float3 camObjDirW_in : TEXCOORD2, 
			   float3 lightVecW_in : TEXCOORD3) : COLOR
{
	// normalizljuk a raszterizcis szakaszban 
	// kiszmtott normlvektort, ugyanis 
	// az interpolci nem eredmnyez felttlenl 
	// egysgvektorokat
	float3 normalWorld = normalize(normalW_in);
	float3 viewWorld = normalize(camObjDirW_in);
	float3 lightWorld = normalize(lightVecW_in);
	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i = max(dot(lightWorld, normalWorld), 0.0f);
	// a diffz szn kiszmtsa
	float3 diffuseColor = i*( gDiffuseMaterialColor * gLightColor).rgb;
	// r felletrl visszavert fny irnynal kiszmtsa
	float3 r = reflect(-lightWorld, normalWorld);
	// a fnyfolt intenzitsnak kiszmtsa
	float i_spec = pow(max(dot(r,viewWorld),0.0f), gSpecularParam);
	// fnyfolt sznnek kiszmtsa
	float3 specColor = i_spec*( gSpecularMaterialColor * gLightColor).rgb;	
	
	float3 texColor = tex2D(SamplerTex0, tex0_in).rgb;
	// kimeneti struktra feltltse
	return float4( (gAmbientMaterialColor + diffuseColor) * texColor + specColor, gDiffuseMaterialColor.a );    
}

technique TexturedTech
{
    pass P0
    {
		CullMode = None;
        // a P0 lps cscspont- s kppontrnyalja
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}