﻿/* Shaderprogramozás - Grafika és játékfejlesztés DirectX-szel ----------------

	Állomány          : CTexturedCube.cpp

    Nyelv             : C++

    Létrehozás dátuma : 2009.01.10

    Utolsó módosítás  : 2009.03.15

    Leírás            : Shaderprogramozás
						Grafika és játékfejlesztés DirectX-szel
						11.7.2. Textúrakoordináta-alapú animáció
						373. oldal

               Copyright (C) Nyisztor Károly 2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Befordított fejállományok -----------------------------------------------*/
#include "CTexturedCube.h"

#include "..\..\common\Path.h"

#include "..\..\common\util\core.h"
using app::util::Core;

#include "..\..\common\util\util.h"
using namespace app::util;
using namespace app::debug;

/*- Névterek ----------------------------------------------------------------*/
namespace app
{
/*!
 * \brief
 * Write brief comment for scene here.
 * 
 * Write detailed description for scene here.
 * 
 * \remarks
 * Write remarks for scene here.
 * 
 * \see
 * Separate items with the '|' character.
 */
namespace scene
{
/*- Definíciók és konstansok ------------------------------------------------*/

/*- Globális változók -------------------------------------------------------*/


/*- Implementációs rész -----------------------------------------------------*/
/*-----------------------------------------------------------------------------
Név		  : CTexturedCube

Leírás	  : konstruktor

Return	  : -
-----------------------------------------------------------------------------*/
CTexturedCube::CTexturedCube( const string& fxFile_in,  
				  const string& textureFile_in, 
				  const D3DXVECTOR3& posVec_in, 
				  const D3DXVECTOR3& rotVec_in, 
				  const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFile_in ),
													 myTexFile(textureFile_in),
													 myPosVec(posVec_in), 
													 myRotVec(rotVec_in), 
													 myScaleVec(scaleVec_in)
{
	myDeclPtr = NULL;
	// D3DDevice mutató lekérdezése
	myD3DDevicePtr = GameCore->Device();
	myCubeVBPtr = NULL;
	myTexturePtr = NULL;
	D3DXMatrixIdentity( &myWorldMat );
	
	D3DXMatrixScaling( &myScaleMat, myScaleVec.x, myScaleVec.y, myScaleVec.z );
	
	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	
	D3DXMatrixTranslation( &myTranslMat, myPosVec.x, myPosVec.y, myPosVec.z );
	
	myAmbientMat = D3DCOLOR_XRGB( 32, 32, 32 );
	myDiffuseMat = D3DCOLOR_XRGB( 255, 255, 255 );
	mySpecularMat = D3DCOLOR_XRGB( 255, 255, 255 );
	myDiffuseLight = D3DCOLOR_XRGB( 255, 255, 255 );
	
	myLightPos = D3DXVECTOR3( 0.f, 10.f, -10.f );	
	// a fényfoltképzõ anyagtulajdonság
	// magasabb értékek élesebb fényfoltokat eredményeznek
	mySpecularPower = 32.0f;
	myTimeSum = 0.f;
	myEffectPtr = NULL;

	CHECK_WEXIT( loadTexture() )
	CHECK_WEXIT( initVertexBuffers() );
	CHECK_WEXIT( initVertexDeclaration() )
	CHECK_WEXIT( loadFX() )

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betöltés és inicializálás. Fájl: %s", fxFile_in.c_str() );
	LOG_INFO( messStr )
}
/*-----------------------------------------------------------------------------
Név		  : ~CTexturedCube

Leírás	  : destruktor

Return	  : -
-----------------------------------------------------------------------------*/
CTexturedCube::~CTexturedCube()
{
	SAFE_RELEASE( myCubeVBPtr );	
	SAFE_RELEASE( myTexturePtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
}

/*!
 * \brief Vertex-struktúra megadása
 * \remarks a shader ez alapján kezeli a csúcspontokat
 */
HRESULT CTexturedCube::initVertexDeclaration()
{
	// vertex deklaráció
	D3DVERTEXELEMENT9 vertexElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		{0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0},
		{0, 24, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0},
		D3DDECL_END()
	};
	CHECK_WEXIT( myD3DDevicePtr->CreateVertexDeclaration( vertexElements, &myDeclPtr ) )

	LOG_INFO( "Vertex deklarálás rendben" );
	return S_OK;
}
/*!
 * \brief
 * textúra képállomány betöltése és hozzárendelése
 * 
 * \remarks
 * bővebben: 363. oldal, "A textúra betöltése" 
 */
HRESULT CTexturedCube::loadTexture()
{
	// mintaállomány betöltése
	if( myTexFile.empty() )
	{
		LOG_ERROR( "Nincs megadva a betöltendô modell neve." )
		return E_FAIL;
	}
	// mintaállomány betöltése
	CHAR texDir[MAX_PATH];
	CHAR textureFile[MAX_PATH];
	
	// grafikákat tartalmazó könyvtár
	strcpy( texDir, GFX_DIR );
	strcpy( textureFile, texDir );	
	strcat( textureFile, myTexFile.c_str() );
	
	CHECK_WEXIT( fileExists(textureFile) )

	CHECK_WEXIT( checkTextureImage( textureFile ) )

	CHECK_WEXIT( D3DXCreateTextureFromFile( myD3DDevicePtr, textureFile, &myTexturePtr ) )

	return S_OK;
}
/*!
 * \brief
 * Ez a függvény ellenôrzi, hogy a betöltött kép alkalmazható-e textúraként az adott videohardveren
 * 
 * \remarks
 * bővebben: 363. oldal, "A textúra betöltése" 
 */
HRESULT CTexturedCube::checkTextureImage(const string& textureFile_in)
{	
	HRESULT hr;
	D3DXIMAGE_INFO imgInfo;
	// információk kinyerése az képbôl
	hr = D3DXGetImageInfoFromFile( textureFile_in.c_str(), &imgInfo );

	UINT width = imgInfo.Width;
	UINT height = imgInfo.Height;
	UINT mipLevels = imgInfo.MipLevels;
	D3DFORMAT format = imgInfo.Format;
	
	// ezen a hardveren használható az adott kép textúraként?
	hr = D3DXCheckTextureRequirements( myD3DDevicePtr, 
						&width,
						&height, 
						&mipLevels,
						0 /*vagy D3DUSAGE_RENDERTARGET*/, 
					   	&format,									   	
						D3DPOOL_DEFAULT );
	// gond van, ha bármelyik paraméter eltér a megadottól
	if( ( width != imgInfo.Width )
		|| ( height != imgInfo.Height )
		|| ( format!= imgInfo.Format )
		|| ( mipLevels != imgInfo.MipLevels ) )
	{
		LOG_ERROR( "A textúra formátuma nem megfelelô:" << textureFile_in );
		return E_FAIL;
	}
	return hr;
}
/*!
 * \brief
 * Vertexadatok deklarálása és feltöltése
 */
HRESULT CTexturedCube::initVertexBuffers()
{	
	// ez a pointer a vertexpuffer adatterületére fog mutatni
	SVertexTex* vertPtr = NULL;
	// 1) kocka
	SVertexTex cubeVerts[] = 
	{
		// elülsô oldal
		{ D3DXVECTOR3(-5.f, 5.f, -5.f),  D3DXVECTOR3(0, 0, -1), D3DXVECTOR2(0, 0) },	// 0 = 14 = 16
		{ D3DXVECTOR3(5.f, 5.f, -5.f),   D3DXVECTOR3(0, 0, -1), D3DXVECTOR2(1, 0) },	// 2 = 4 = 17
		{ D3DXVECTOR3(-5.f, -5.f, -5.f), D3DXVECTOR3(0, 0, -1), D3DXVECTOR2(0, 1) },	// 1 = 15 = 18

		{ D3DXVECTOR3(-5.f, -5.f, -5.f), D3DXVECTOR3(0, 0, -1), D3DXVECTOR2(0, 1) },	// 1 = 15 = 18
		{ D3DXVECTOR3(5.f, 5.f, -5.f), D3DXVECTOR3(0, 0, -1), D3DXVECTOR2(1, 0) },	// 2 = 4 = 17
		{ D3DXVECTOR3(5.f, -5.f, -5.f), D3DXVECTOR3(0, 0, -1), D3DXVECTOR2(1, 1) },	// 3 = 5 = 19
		// jobb oldal
		{ D3DXVECTOR3(5.f, 5.f, -5.f), D3DXVECTOR3(1, 0, 0), D3DXVECTOR2(0, 0) },	// 4 = 2
		{ D3DXVECTOR3(5.f, 5.f, 5.f), D3DXVECTOR3(1, 0, 0), D3DXVECTOR2(1, 0) },		// 6 = 8
		{ D3DXVECTOR3(5.f, -5.f, -5.f), D3DXVECTOR3(1, 0, 0), D3DXVECTOR2(0, 1) },	// 5

		{ D3DXVECTOR3(5.f, -5.f, -5.f), D3DXVECTOR3(1, 0, 0), D3DXVECTOR2(0, 1) },	// 5
		{ D3DXVECTOR3(5.f, 5.f, 5.f), D3DXVECTOR3(1, 0, 0), D3DXVECTOR2(1, 0) },		// 6 = 8
		{ D3DXVECTOR3(5.f, -5.f, 5.f), D3DXVECTOR3(1, 0, 0), D3DXVECTOR2(1, 1) },	// 7 = 9
		// hátsó oldal
		{ D3DXVECTOR3(5.f, 5.f, 5.f), D3DXVECTOR3(0, 0, 1), D3DXVECTOR2(0, 0) },		// 8
		{ D3DXVECTOR3(-5.f, 5.f, 5.f), D3DXVECTOR3(0, 0, 1), D3DXVECTOR2(1, 0) },	// 10 = 12
		{ D3DXVECTOR3(5.f, -5.f, 5.f), D3DXVECTOR3(0, 0, 1), D3DXVECTOR2(0, 1) },	// 9

		{ D3DXVECTOR3(5.f, -5.f, 5.f), D3DXVECTOR3(0, 0, 1), D3DXVECTOR2(0, 1) },	// 9
		{ D3DXVECTOR3(-5.f, 5.f, 5.f), D3DXVECTOR3(0, 0, 1), D3DXVECTOR2(1, 0) },	// 10 = 12
		{ D3DXVECTOR3(-5.f, -5.f, 5.f), D3DXVECTOR3(0, 0, 1), D3DXVECTOR2(1, 1) },	// 11 = 13
		// bal oldal
		{ D3DXVECTOR3(-5.f, 5.f, 5.f), D3DXVECTOR3(-1, 0, 0), D3DXVECTOR2(0, 0) },	// 12
		{ D3DXVECTOR3(-5.f, 5.f, -5.f), D3DXVECTOR3(-1, 0, 0), D3DXVECTOR2(1, 0) },	// 14
		{ D3DXVECTOR3(-5.f, -5.f, 5.f), D3DXVECTOR3(-1, 0, 0), D3DXVECTOR2(0, 1) },	// 13

		{ D3DXVECTOR3(-5.f, -5.f, 5.f), D3DXVECTOR3(-1, 0, 0), D3DXVECTOR2(0, 1) },	// 13
		{ D3DXVECTOR3(-5.f, 5.f, -5.f), D3DXVECTOR3(-1, 0, 0), D3DXVECTOR2(1, 0) },	// 14
		{ D3DXVECTOR3(-5.f, -5.f, -5.f), D3DXVECTOR3(-1, 0, 0), D3DXVECTOR2(1, 1) },	// 15
		// kocka teteje
		{ D3DXVECTOR3(-5.f, 5.f, 5.f), D3DXVECTOR3(0, 1, 0), D3DXVECTOR2(0, 0) },	// 12
		{ D3DXVECTOR3(5.f, 5.f, 5.f), D3DXVECTOR3(0, 1, 0), D3DXVECTOR2(1, 0) },		// 6 = 8
		{ D3DXVECTOR3(-5.f, 5.f, -5.f), D3DXVECTOR3(0, 1, 0), D3DXVECTOR2(0, 1) },	// 16

		{ D3DXVECTOR3(-5.f, 5.f, -5.f), D3DXVECTOR3(0, 1, 0), D3DXVECTOR2(0, 1) },	// 16
		{ D3DXVECTOR3(5.f, 5.f, 5.f), D3DXVECTOR3(0, 1, 0), D3DXVECTOR2(1, 0) },		// 6 = 8		
		{ D3DXVECTOR3(5.f, 5.f, -5.f), D3DXVECTOR3(0, 1, 0), D3DXVECTOR2(1, 1) },	// 17
		// kocka alja
		{ D3DXVECTOR3(-5.f, -5.f, -5.f), D3DXVECTOR3(0, -1, 0), D3DXVECTOR2(0, 0) },	// 18
		{ D3DXVECTOR3(5.f, -5.f, -5.f), D3DXVECTOR3(0, -1, 0), D3DXVECTOR2(1, 0) },	// 19
		{ D3DXVECTOR3(-5.f, -5.f, 5.f), D3DXVECTOR3(0, -1, 0), D3DXVECTOR2(0, 1) },	// 13

		{ D3DXVECTOR3(-5.f, -5.f, 5.f), D3DXVECTOR3(0, -1, 0), D3DXVECTOR2(0, 1) },	// 13
		{ D3DXVECTOR3(5.f, -5.f, -5.f), D3DXVECTOR3(0, -1, 0), D3DXVECTOR2(1, 0) },	// 19
		{ D3DXVECTOR3(5.f, -5.f, 5.f), D3DXVECTOR3(0, -1, 0), D3DXVECTOR2(1, 1) },	// 7 = 9
	};

	// vertex puffer létrehozása
	CHECK_WEXIT( myD3DDevicePtr->CreateVertexBuffer( 36 * sizeof( SVertexTex ),
													0, 
													0,
													D3DPOOL_DEFAULT, 
													&myCubeVBPtr,
													NULL ) )
	
	vertPtr = NULL;
	// megszerezzük majd zároljuk a vertex puffer memóriaterületét, hogy átmásolhassuk a vertex adatokat	
	if( FAILED( myCubeVBPtr->Lock( 0, sizeof( cubeVerts ), (VOID**)&vertPtr, 0 ) ) )
	{
		return E_FAIL;
	}

	// vertex adatok másolása
	memcpy( vertPtr, cubeVerts, sizeof( cubeVerts ) );

	// vertex puffer zárolásának feloldása
	myCubeVBPtr->Unlock();

	return S_OK;
}
/*!
 * \brief
 * Effekt-betöltő
 * 
 * \returns
 * S_OK, ha minden rendben
 */
HRESULT CTexturedCube::loadFX()
{
	CHECK_WEXIT(fileExists(myFXFile));	
	// effekt létrehozása fájból
	ID3DXBuffer* errorBufferPtr = NULL;
	
	CHECK_WLOG( D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0,/*D3DXSHADER_DEBUG*/
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							) );
	// effekt betöltési hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		return E_FAIL;
	}

	// kezelõk lekérdezése
	myHTech       = myEffectPtr->GetTechniqueByName("TexturedTech");
	myHWVP        = myEffectPtr->GetParameterByName(0, "gWorldViewProjection");
	myHAmbientMaterialColor	= myEffectPtr->GetParameterByName(0, "gAmbientMaterialColor");
	myHDiffuseMaterialColor = myEffectPtr->GetParameterByName(0, "gDiffuseMaterialColor");
	myHSpecularMaterialColor = myEffectPtr->GetParameterByName(0, "gSpecularMaterialColor");
	myHSpecularPower = myEffectPtr->GetParameterByName(0, "gSpecularParam" );
	
	myHWorld 	= myEffectPtr->GetParameterByName(0, "gWorld");
	myHLightPos = myEffectPtr->GetParameterByName(0, "gLightPos");
	myHCameraPos = myEffectPtr->GetParameterByName(0, "gCameraPos");
	
	myHLightColor = myEffectPtr->GetParameterByName(0, "gLightColor");
	myHTime = myEffectPtr->GetParameterByName(0, "gTime");

	myHTex0 = myEffectPtr->GetParameterByName(0, "gTex0");

	if( !myHTech
		|| !myHWVP
		|| !myHAmbientMaterialColor
		|| !myHDiffuseMaterialColor
		|| !myHSpecularMaterialColor
		|| !myHSpecularPower
		|| !myHWorld
		|| !myHLightPos
		|| !myHCameraPos
		|| !myHLightColor
		|| !myHTex0
		|| !myHTime )
	{
		LOG_ERROR( "Effekt paraméter beolvasási hiba!" );
		return E_FAIL;
	}

	CHECK_WEXIT( myEffectPtr->SetFloat( myHSpecularPower, mySpecularPower ) );

	return S_OK;
}

void CTexturedCube::update( float dt )
{
	
	myTimeSum += dt;

	//myTimeSum = myTimeSum > 1.f ? myTimeSum - 1.f : myTimeSum;
	// lassan forgatjuk a modellt a saját tengelye körül
	/*
	static const D3DXVECTOR3 rotationStep( 0.f, 0.1f, 0 );
	myRotVec += rotationStep * dt;

	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	*/
}
/*-----------------------------------------------------------------------------
Név		 : render

Leírás	  : Ez a függvény a megjelenítésért felelôs

Return	  : -
-----------------------------------------------------------------------------*/
void CTexturedCube::render()
{
	myEffectPtr->SetTechnique(myHTech);
	myD3DDevicePtr->SetStreamSource( 0, myCubeVBPtr, 0, sizeof(SVertexTex) );
	myD3DDevicePtr->SetVertexDeclaration(myDeclPtr);

	myEffectPtr->SetFloat( myHTime, myTimeSum );
	myEffectPtr->SetMatrix(myHWVP, &( getWorldMat() * GameCore->Camera()->getViewProjMatrix() ));
		
	myEffectPtr->SetValue(myHAmbientMaterialColor, &myAmbientMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHDiffuseMaterialColor, &myDiffuseMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHSpecularMaterialColor, &mySpecularMat, sizeof(D3DXCOLOR));
	
	myEffectPtr->SetValue(myHLightPos, &myLightPos, sizeof(D3DXVECTOR3));
	myEffectPtr->SetValue(myHLightColor, &myDiffuseLight, sizeof(D3DXCOLOR));
		
	myEffectPtr->SetMatrix(myHWorld, &getWorldMat());

	myEffectPtr->SetValue( myHCameraPos, &( GameCore->Camera()->getPosition() ), sizeof(D3DXVECTOR3) );
	
	UINT passCnt = 0;
	myEffectPtr->Begin( &passCnt, 0 );
	for( UINT p = 0; p < passCnt; ++p )
	{
		myEffectPtr->BeginPass( p );
        myEffectPtr->SetTexture( myHTex0, myTexturePtr );
		myD3DDevicePtr->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 12 );

		myEffectPtr->EndPass();
	}
	myEffectPtr->End();
}

const D3DXMATRIX& CTexturedCube::getWorldMat() 
{		
		myWorldMat = myScaleMat * myRotMat * myTranslMat;
		return myWorldMat;
}


void CTexturedCube::onLostDevice()
{
	myEffectPtr->OnLostDevice();
	SAFE_RELEASE( myCubeVBPtr );
}

void CTexturedCube::onResetDevice()
{
	CHECK_WEXIT(initVertexBuffers());
	myEffectPtr->OnResetDevice();
}
}
}