/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: MultiTextured.fx
	Lers	: multitextrzott modell, direkcionlis megvilgtssal
	Lsd	: 11.7.4. "Multitextrzs"
			  377. oldal
------------------------------------------------- */
uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldViewProjection;
uniform extern float4 gAmbientMaterialColor;
uniform extern float4 gDiffuseMaterialColor;
uniform extern float4 gLightColor;
uniform extern float3 gLightPos;

uniform extern texture gTex0;
uniform extern texture gTex1;
uniform extern texture gTex2;
uniform extern texture gTex3;

sampler SamplerTex0 = sampler_state
{
	Texture = <gTex0>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};
sampler SamplerTex1 = sampler_state
{
	Texture = <gTex1>;
	MinFilter = ANISOTROPIC;
	MagFilter = ANISOTROPIC;
	MipFilter = ANISOTROPIC;
	AddressU  = WRAP;
    AddressV  = WRAP;
};
sampler SamplerTex2 = sampler_state
{
	Texture = <gTex2>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};
sampler SamplerTex3 = sampler_state
{
	Texture = <gTex3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

struct VS_OUTPUT
{
    float4 PositionProj : POSITION0;
	float2 Tex0			: TEXCOORD0;
	float3 NormalW 		: TEXCOORD1;
	float3 PosL : TEXCOORD2;
	float4 DiffuseColor	: COLOR0;
};
// cscspont rnyal
VS_OUTPUT vs_main(float3 posLocal_in : POSITION0, 
				  float3 normalLocal_in : NORMAL0, 
				  float2 tex0_in : TEXCOORD0)
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;
	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(posLocal_in, 1.0f), gWorldViewProjection);

	outVS.Tex0 = tex0_in * 8.0f;
	outVS.NormalW = normalLocal_in;
	
	outVS.PosL = posLocal_in;//posW.y;
	
	float3 posW = mul(float4(posLocal_in, 1.0f), gWorld).xyz;
	float3 lightVecW = normalize( gLightPos - posW );
	// felleti normlvektor transzformlsa modelltrbl a vilg-koordintarendszerbe
	float3 normalW = mul(float4(normalLocal_in, 0.0f), gWorld).xyz;	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i_diffuse = max(dot(lightVecW, normalW), 0.0f);
	// a diffz szn kiszmtsa
	outVS.DiffuseColor = i_diffuse * ( gDiffuseMaterialColor * gLightColor);	
	
	// a kitlttt struktra visszatrtse
    return outVS;
}
// magassg-intervallumok
const float L1 = 0.5f;  // fld
const float L2 = 2.0f;  // f
const float L3 = 10.0f; // h

// kppont rnyal
float4 ps_main(float4 color_in : COLOR0,
			   float2 tex0_in : TEXCOORD0, 
			   float3 normalW_in : TEXCOORD1,
			   float3 posL_in : TEXCOORD2
			   ) : COLOR
{
	// normalizljuk a raszterizcis szakaszban 
	// kiszmtott normlvektort, ugyanis 
	// az interpolci nem eredmnyez felttlenl 
	// egysgvektorokat
	float3 normalWorld = normalize(normalW_in);
	// texelszn lekrdezse
	float3 tex0Color = tex2D(SamplerTex0, tex0_in).rgb;
	float3 tex1Color = tex2D(SamplerTex1, tex0_in).rgb;
	float3 tex2Color = tex2D(SamplerTex2, tex0_in).rgb;
	float3 tex3Color = tex2D(SamplerTex3, tex0_in).rgb;
		
	// a normlvektor alapjn kiszmtjuk, mennyire ferde a fellet
	// a meredeksg slyozza, milyen mrtkben szikls a fellet
	//float slope = pow( max(dot(float3(0.0f, 1.0f, 0.0f), normalWorld), 0.0f), 2.0f );
	float slope = max(dot(float3(0.0f, 1.0f, 0.0f), normalWorld), 0.0f);
	
	float3 texGroundColor = float3(0,0,0);
	if(posL_in.y < L1)
	{
		texGroundColor = tex0Color * slope;
	}
	else if(posL_in.y < L2)
	{
		float weight = (posL_in.y - L1) / (L2-L1);
		//weight = pow(weight,2.0f);
		texGroundColor = tex1Color * weight * slope + tex0Color * (1.0f - weight) * slope;		
	}
	else 
	{
		float weight = (posL_in.y - L2) / (L3-L2);
		//weight = pow(weight,3.0f);
		weight = saturate(weight);
		texGroundColor = tex3Color * weight * slope + tex1Color * (1.0f - weight) * slope;
	}		
	
	//texGroundColor = tex0Color * slope;
	// szikla textra
	tex2Color *= (1.0f - slope);
	// textrkbl kombinlt sznrtk
	float3 texColor = texGroundColor + tex2Color;
	
	// kppont sznnek kiszmtsa az ambiens anyagszn s a diffz fnyvisszaverds-,
	// valamint a textrk alapjn kiszmtott sznrtk alapjn
	return float4( (gAmbientMaterialColor + color_in) * texColor, gDiffuseMaterialColor.a );    
}

technique TexturedTech
{
    pass P0
    {
        // a P0 lps cscspont- s kppontrnyalja
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}