/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: Alfablend.fx
	Lsd	: 11.8. "tltszsg. Az anyag s a textra alfacsatorna alkalmazsa"
			  385. oldal
------------------------------------------------- */

uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldViewProjection;
uniform extern float4 gAmbientMaterialColor;
uniform extern float4 gDiffuseMaterialColor;
uniform extern float4 gSpecularMaterialColor;
uniform extern float4 gLightColor;
uniform extern float3 gLightVecW;
uniform extern float3 gCameraPos;

uniform extern float gSpecularParam;

uniform extern texture gTex0;
 
struct VS_OUTPUT
{
    float4 PositionProj : POSITION0;
	float2 Tex0    		: TEXCOORD0;
	float3 PositionWorld: TEXCOORD1;
    float3 NormalWorld 	: TEXCOORD2;	
};
// cscspont rnyal
VS_OUTPUT vs_main(float3 posLocal_in : POSITION0, float3 normalLocal_in : NORMAL0, float2 tex0_in: TEXCOORD0)
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;
	
	// cscspont helyzetnek transzformcija vilgkoordinta-rendszerbe
	outVS.PositionWorld = mul(float4(posLocal_in, 1.0f), gWorld).xyz;

	// felleti normlvektor transzformlsa modelltrbl a vilg-koordintarendszerbe
	float3 normalW = mul(float4(normalLocal_in, 0.0f), gWorld).xyz;
	outVS.NormalWorld = normalize(normalW);

	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(posLocal_in, 1.0f), gWorldViewProjection);
	
	// textra-koordintk tadsa a raszterizcis szakasznak
	outVS.Tex0 = tex0_in;
	
	// a kitlttt struktra visszatrtse
    return outVS;
}

sampler Sampler = sampler_state
{
	Texture = <gTex0>;
	MinFilter = Anisotropic;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	MaxAnisotropy = 8;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

// kppont rnyal
float4 ps_main(float2 tex0_in : TEXCOORD0, float3 positionWorld_in : TEXCOORD1, float3 normalWorld_in : TEXCOORD2) : COLOR
{
	// normalizljuk a raszterizcis szakaszban 
	// kiszmtott normlvektort, ugyanis 
	// az interpolci nem eredmnyez felttlenl 
	// egysgvektorokat
	normalWorld_in = normalize(normalWorld_in);
	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i = max(dot(gLightVecW, normalWorld_in), 0.0f);
	// a diffz szn kiszmtsa
	float3 diffuseColor = i*( gDiffuseMaterialColor * gLightColor).rgb;
	
	// c (nzeti irny) vektor kiszmtsa
	float3 c = normalize(gCameraPos - positionWorld_in);
	// r (felletrl visszavert fny) kiszmtsa
	float3 r = reflect(-gLightVecW, normalWorld_in);
	// a fnyfolt intenzitsnak kiszmtsa
	float i_spec = pow(max(dot(r,c),0.0f), gSpecularParam);
	// fnyfolt sznnek kiszmtsa
	float3 specColor = i_spec*( gSpecularMaterialColor * gLightColor).rgb;
	
	float4 texColor = tex2D(Sampler, tex0_in);
	// kimeneti struktra feltltse
	return float4( gAmbientMaterialColor + diffuseColor.rgb * texColor + specColor, texColor.a * gDiffuseMaterialColor.a );    
}

technique AlfaTech
{
    pass P0
    {
		AlphaBlendEnable = TRUE;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		BlendOp = ADD;
	//CullMode = None;

        // a P0 lps cscspont- s kppontrnyalja
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}
