/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : CTexturedMeshNode.cpp

    Nyelv             : C++

    Ltrehozs dtuma : 2009.01.10

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.7.3. Textrzott X-modell
						375. oldal

               Copyright (C) Nyisztor Kroly 2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Szksges fggvnyknyvtrak --------------------------------------------*/
// biztostsuk a szksges fggvnyknyvtrak linkelst!

/*- Befordtott fejllomnyok -----------------------------------------------*/
#include "CTexturedMeshNode.h"

#include "..\..\common\scene\BaseMesh.h"

#include "..\..\common\util\core.h"
using app::util::Core;

#include "..\..\common\util\util.h"
using namespace app::util;
using namespace app::debug;

/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
/*!
 * \brief
 * Write brief comment for scene here.
 * 
 * Write detailed description for scene here.
 * 
 * \remarks
 * Write remarks for scene here.
 * 
 * \see
 * Separate items with the '|' character.
 */
namespace scene
{

/*- Defincik s konstansok ------------------------------------------------*/
// statikus adattagok inicializlsa
/*- Globlis vltozk -------------------------------------------------------*/

/*- Implementcis rsz -----------------------------------------------------*/

CTexturedMeshNode::CTexturedMeshNode( const string& fxFileName_in, 
				  const string& meshFile_in,
				  const D3DCOLOR diffuseColor_in,
				  const D3DXVECTOR3& posVec_in, 
				  const D3DXVECTOR3& rotVec_in, 
				  const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFileName_in ), 
													 myDiffuseMat( diffuseColor_in ),
													 myPosVec(posVec_in), 
													 myRotVec(rotVec_in), 
													 myScaleVec(scaleVec_in)
{
	myDeclPtr = NULL;

	myMeshPtr = new BaseMesh( meshFile_in );

	// D3DDevice mutat lekrdezse
	myD3DDevicePtr = GameCore->Device();
	if( !myD3DDevicePtr )
	{
		const wchar_t errorStr[] = L"Nincs rvnyes Direct3D eszkz mutat!";
		::MessageBoxW(0, errorStr, 0, 0);
		return;
	}

	D3DXMatrixIdentity( &myWorldMat );
	
	D3DXMatrixScaling( &myScaleMat, myScaleVec.x, myScaleVec.y, myScaleVec.z );
	
	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	
	D3DXMatrixTranslation( &myTranslMat, myPosVec.x, myPosVec.y, myPosVec.z );
	
	myAmbientMat = D3DCOLOR_XRGB( 16, 16, 16 );
	
	mySpecularMat = D3DCOLOR_XRGB( 64, 64, 255 );
	myDiffuseLight = D3DCOLOR_XRGB( 255, 255, 255 );
	
	myLightVec = D3DXVECTOR3( -0.5f, 0.5f, -1.f );
	D3DXVec3Normalize( &myLightVec, &myLightVec );
	// a fnyfoltkpz anyagtulajdonsg
	// magasabb rtkek lesebb fnyfoltokat eredmnyeznek
	mySpecularPower = 16.0f;

	myEffectPtr = NULL;
	
	CHECK_WEXIT( initVertexDeclaration() );

	CHECK_WEXIT( loadFX() );

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betlts s inicializls. Fjl: %s", fxFileName_in.c_str() );
	LOG_INFO( messStr )	
}

CTexturedMeshNode::~CTexturedMeshNode()
{
	SAFE_RELEASE( myMeshPtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
}

/*!
 * \brief Vertex-struktra megadsa
 * \remarks a shader ez alapjn kezeli a cscspontokat
 */
HRESULT CTexturedMeshNode::initVertexDeclaration()
{
	// vertex deklarci
	D3DVERTEXELEMENT9 vertexElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		{0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0},
		{0, 24, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0},
		D3DDECL_END()
	};
	CHECK_WEXIT( myD3DDevicePtr->CreateVertexDeclaration( vertexElements, &myDeclPtr) );

	LOG_INFO( "Vertex deklarls rendben" );
	myMeshPtr->setVertexDecl( vertexElements );

	return S_OK;
}


HRESULT CTexturedMeshNode::loadFX()
{
	CHECK_WEXIT(fileExists(myFXFile));
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	
	D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0,//D3DXSHADER_DEBUG, 
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							);
	// effekt betltsi hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		return E_FAIL;
	}

	// kezelk lekrdezse
	mhTech       = myEffectPtr->GetTechniqueByName("AlfaTech");
	myHWVP        = myEffectPtr->GetParameterByName(0, "gWorldViewProjection");
	myHAmbientMaterialColor	= myEffectPtr->GetParameterByName(0, "gAmbientMaterialColor");
	myHDiffuseMaterialColor = myEffectPtr->GetParameterByName(0, "gDiffuseMaterialColor");
	myHSpecularMaterialColor = myEffectPtr->GetParameterByName(0, "gSpecularMaterialColor");
	myHSpecularPower = myEffectPtr->GetParameterByName(0, "gSpecularParam" );
	myHTexture = myEffectPtr->GetParameterByName(0, "gTex0" );
	
	myHWorld 	= myEffectPtr->GetParameterByName(0, "gWorld");
	myHLightVec = myEffectPtr->GetParameterByName(0, "gLightVecW");
	myHCameraPos = myEffectPtr->GetParameterByName(0, "gCameraPos");
	
	myHLightColor = myEffectPtr->GetParameterByName(0, "gLightColor");	

	if( !mhTech
		|| !myHWVP
		|| !myHAmbientMaterialColor
		|| !myHDiffuseMaterialColor
		|| !myHSpecularMaterialColor
		|| !myHSpecularPower
		|| !myHTexture
		|| !myHWorld
		|| !myHLightVec
		|| !myHLightColor	)
	{
		LOG_ERROR( "Effekt paramter beolvassi hiba!" );
		return E_FAIL;
	}

	myEffectPtr->SetFloat( myHSpecularPower, mySpecularPower );

	return S_OK;
}

const D3DXMATRIX& CTexturedMeshNode::getWorldMat() 
{		
		myWorldMat = myScaleMat * myRotMat * myTranslMat;
		return myWorldMat;
}

void CTexturedMeshNode::update( float dt )
{
	// lassan forgatjuk a modellt a sajt tengelye krl
	/*static const D3DXVECTOR3 rotationStep( 0.f, 0.01f, 0 );
	myRotVec += rotationStep * dt;

	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	*/
}

void CTexturedMeshNode::render()
{
	myEffectPtr->SetTechnique(mhTech);

	myEffectPtr->SetMatrix(myHWVP, &( getWorldMat() * GameCore->Camera()->getViewProjMatrix() ));
		
	myEffectPtr->SetValue(myHAmbientMaterialColor, &myAmbientMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHDiffuseMaterialColor, &myDiffuseMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHSpecularMaterialColor, &mySpecularMat, sizeof(D3DXCOLOR));
	
	myEffectPtr->SetValue(myHLightVec, &myLightVec, sizeof(D3DXVECTOR3));
	myEffectPtr->SetValue(myHLightColor, &myDiffuseLight, sizeof(D3DXCOLOR));
		
	myEffectPtr->SetMatrix(myHWorld, &getWorldMat());

	myEffectPtr->SetValue( myHCameraPos, &( GameCore->Camera()->getPosition() ), sizeof(D3DXVECTOR3) );
	
	myMeshPtr->render( myEffectPtr, myHTexture, NULL /* no mat */, true /* draw opaque subsets */ );
}

void CTexturedMeshNode::onLostDevice()
{
	myEffectPtr->OnLostDevice();
}

void CTexturedMeshNode::onResetDevice()
{
	myEffectPtr->OnResetDevice();
}

}
}

