/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: BumpMap.fx
	Lers	: Buckalekpzs, textratrbeli normlvektorokkal
	Lsd	: 11.9.2. "Buckalekpzs"
			  397. oldal
------------------------------------------------- */
uniform extern float4x4 gWorldViewProjection;
uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldInverse;
uniform extern float4 	gAmbientMaterialColor;
uniform extern float4 	gDiffuseMaterialColor;
uniform extern float4 	gSpecularMaterialColor;
uniform extern float4 	gLightColor;
uniform extern float3 	gLightPos;
uniform extern float3   gCameraPos;
uniform extern texture  gTex0;
uniform extern texture  gBumpMap;

uniform extern float gSpecularParam;

sampler SamplerTex0 = sampler_state
{
	Texture = <gTex0>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler SamplerNormalMap = sampler_state
{
	Texture = <gBumpMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

struct VS_OUTPUT 
{
    float4 PositionProj   	: POSITION0;
	float2 Tex0	   			: TEXCOORD0;
    float3 camObjDirTangent : TEXCOORD1;
    float3 lightDirTangent 	: TEXCOORD2;
};

VS_OUTPUT vs_main( float3 posLocal_in      : POSITION0, 
					  float2 tex0_in      	  : TEXCOORD0,
                      float3 tangentLocal_in  : TANGENT0,
                      float3 binormalLocal_in : BINORMAL0,
                      float3 normalLocal_in   : NORMAL0 )
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;
	
	// a textratr objektumtrben kifefejezett TBN-bzisa 
	// segtsgvel vgezhet el a textratr->modeltr koordintarendszer transzformci,
	// inverzvel pedig a msik irnyba transzformlhatunk a kt koordintarendszer kztt
	// az rtkeket a mesh betltsekor generljuk az alkalmazskdban
	float3x3 TBN;
	TBN[0] = tangentLocal_in;
	TBN[1] = binormalLocal_in;
	TBN[2] = normalLocal_in;
		
	// a transzformcis mtrix, amellyel a textratrbl a modellezsi koordintarendszerbe alaktunk
	//    | tangens  |
	// M =| binorml |
	//    | norml   |
	// az objektumtrbl a textratrbe a TBN mtrix inverzvel transzformlhatunk
	// mivel ortogonlis, az inverze egyenl a transzponltjval
	float3x3 matObj2TangentSpace = transpose(TBN);
	
	// a kamera-objektum irny s a fny helyzett kzs koordintarendszerbe kell hozni a textratrbeli normlvektorokkal
	// ehhez elbb egy vilg-modelltr, majd egy modelltr-textratr transzformcit kell vgrehajtani
	
	// kamera objektumtrbeli helyzetnek kiszmtsa
	float3 cameraPosLocal = mul(float4(gCameraPos, 1.0f), gWorldInverse);
	
	// objektum-kamera vektor kiszmtsa modelltrben
	float3 camObjDirLocal = cameraPosLocal - posLocal_in;
	
	// kamera-objektum irny objektumtr->textratr transzformci	
	outVS.camObjDirTangent = mul(camObjDirLocal, matObj2TangentSpace);

	// cscspont helyzetnek transzformcija vilgkoordinta-rendszerbe
	float3 posW = mul(float4(posLocal_in, 1.0f), gWorld).xyz;
	// fnyvektor kiszmtsa
	float3 lightVecW = normalize( gLightPos - posW );
	// fnyvektor vilg->objektumtr transzformci
	float3 lightDirL = mul(float4(lightVecW, 0.0f), gWorldInverse).xyz;
	// fnyvektor objektumtr->textratr transzformci
	outVS.lightDirTangent  = mul(lightDirL, matObj2TangentSpace);
	
	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(posLocal_in, 1.0f), gWorldViewProjection);
	
	// textra-koordintk tadsa a raszterizcis szakasznak
	outVS.Tex0 = tex0_in;
	
	// a kitlttt struktra visszatrtse
    return outVS;
}
// kppont rnyal
float4 ps_main( float2 tex0_in : TEXCOORD0,
				float3 camObjDirTangent_in : TEXCOORD1,
                float3 lightDirTangent_in : TEXCOORD2 ) : COLOR
{
	// normalizljuk a raszterizcis szakaszban 
	// kiszmtott vektorokat, ugyanis 
	// az interpolci nem eredmnyez felttlenl 
	// egysgvektorokat
	camObjDirTangent_in = normalize(camObjDirTangent_in);
	lightDirTangent_in 	= normalize(lightDirTangent_in);
		
	// texel lekrdezse a normal map-bl
	float3 normalTangent = tex2D(SamplerNormalMap, tex0_in);
	
	// (eljeltelen) [0, 1] intervallumbl [-1, 1]-re alaktunk.
    normalTangent = 2.0f * normalTangent - 1.0f;
    normalTangent = normalize(normalTangent);
	
	// a felletrl visszavert fny irnynak kiszmtsa
	float3 r = reflect(-lightDirTangent_in, normalTangent);
	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i_diffuse = max(dot(lightDirTangent_in, normalTangent), 0.0f);
	
	// a fnyfolt intenzitsnak kiszmtsa
	float i_spec = pow(max(dot(r, camObjDirTangent_in),0.0f), gSpecularParam);
	
	if(i_diffuse <= 0.0f)
	     i_spec = 0.0f;
	
	// a diffz szn kiszmtsa
	float3 diffuseColor = i_diffuse * ( gDiffuseMaterialColor * gLightColor).rgb;

	// fnyfolt sznnek kiszmtsa
	float3 specColor = i_spec*( gSpecularMaterialColor * gLightColor).rgb;	
	
	// texel lekrdezse
	float4 texColor = tex2D(SamplerTex0, tex0_in);
	
	// kimeneti struktra feltltse
	return float4( (gAmbientMaterialColor + diffuseColor) * texColor + specColor, gDiffuseMaterialColor.a );
}

technique BumpMapTech
{
    pass P0
    {
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}