/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : CBumpMappedMeshNode.cpp

    Nyelv             : C++

    Ltrehozs dtuma : 2009.01.26

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.9.2. "Buckalekpzs"
						397. oldal

               Copyright (C) Nyisztor Kroly 2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Szksges fggvnyknyvtrak --------------------------------------------*/
// biztostsuk a szksges fggvnyknyvtrak linkelst!

/*- Befordtott fejllomnyok -----------------------------------------------*/
#include "CBumpMappedMeshNode.h"

#include "..\..\common\scene\BaseMesh.h"

#include "..\..\common\Path.h"

#include "..\..\common\util\core.h"
using app::util::Core;

#include "..\..\common\util\util.h"
using namespace app::util;
using namespace app::debug;

/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
/*!
 * \brief
 * Write brief comment for scene here.
 * 
 * Write detailed description for scene here.
 * 
 * \remarks
 * Write remarks for scene here.
 * 
 * \see
 * Separate items with the '|' character.
 */
namespace scene
{

/*- Defincik s konstansok ------------------------------------------------*/
// statikus adattagok inicializlsa
/*- Globlis vltozk -------------------------------------------------------*/

/*- Implementcis rsz -----------------------------------------------------*/

CBumpMappedMeshNode::CBumpMappedMeshNode( const string& fxFileName_in, 
				  const string& meshFile_in,
				  const string& bumpMapFile_in,
				  const D3DXVECTOR3& posVec_in, 
				  const D3DXVECTOR3& rotVec_in, 
				  const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFileName_in ), 
													 myBumpMapFile( bumpMapFile_in ),
													 myPosVec(posVec_in), 
													 myRotVec(rotVec_in), 
													 myScaleVec(scaleVec_in)
{
	myDeclPtr = NULL;

	myMeshPtr = new BaseMesh( meshFile_in );

	// D3DDevice mutat lekrdezse
	myD3DDevicePtr = GameCore->Device();

	D3DXMatrixIdentity( &myWorldMat );
	
	D3DXMatrixScaling( &myScaleMat, myScaleVec.x, myScaleVec.y, myScaleVec.z );
	
	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
	
	D3DXMatrixTranslation( &myTranslMat, myPosVec.x, myPosVec.y, myPosVec.z );
	
	myAmbientMat = D3DCOLOR_XRGB( 16, 16, 16 );
	myDiffuseMat = D3DCOLOR_XRGB( 128, 128, 128 );
	mySpecularMat = D3DCOLOR_XRGB( 64, 32, 32 );
	myDiffuseLight = D3DCOLOR_XRGB( 255, 255, 255 );
	
	myLightPos = D3DXVECTOR3( 20.f, 20.f, 80.f );
	// a fnyfoltkpz anyagtulajdonsg
	// magasabb rtkek lesebb fnyfoltokat eredmnyeznek
	mySpecularPower = 1.0f;

	myEffectPtr = NULL;
	
	CHECK_WEXIT( loadBumpTexture() );
	
	CHECK_WEXIT( initVertexDeclaration() );

	CHECK_WEXIT( loadFX() );

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betlts s inicializls. Fjl: %s", fxFileName_in.c_str() );
	LOG_INFO( messStr )	
}

CBumpMappedMeshNode::~CBumpMappedMeshNode()
{
	SAFE_RELEASE( myMeshPtr );
	SAFE_RELEASE( myBumpMapPtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
}

/*!
 * \brief Vertex-struktra megadsa
 * \remarks a shader ez alapjn kezeli a cscspontokat
 */
HRESULT CBumpMappedMeshNode::initVertexDeclaration()
{
	// vertex deklarci
	D3DVERTEXELEMENT9 vertexElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		{0, 12, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0},
		{0, 20, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT, 0},
		{0, 32, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0},
		{0, 44, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0},
		D3DDECL_END()
	};
	CHECK_WEXIT( myD3DDevicePtr->CreateVertexDeclaration( vertexElements, &myDeclPtr) );

	LOG_INFO( "Vertex deklarls rendben" );
	myMeshPtr->setVertexDecl( vertexElements, true, true );

	return S_OK;
}

/*!
 * \brief
 * textra kpllomny betltse s hozzrendelse
 * 
 * \returns
 * S_OK, ha minden rendben
 */
HRESULT CBumpMappedMeshNode::loadBumpTexture()
{
	// mintallomny betltse
	if( myBumpMapFile.empty() )
	{
		LOG_ERROR( "Nincs megadva a betltend textra neve." )
		return E_FAIL;
	}
	// mintallomny betltse
	CHAR texDir[MAX_PATH];
	CHAR textureFile[MAX_PATH];
	
	// grafikkat tartalmaz knyvtr
	strcpy( texDir, GFX_DIR );
	strcpy( textureFile, texDir );	
	strcat( textureFile, myBumpMapFile.c_str() );

	// ltezik a fjl? - megprbljuk megnyitni
	CHECK_WEXIT( fileExists(textureFile) )

	CHECK_WEXIT( D3DXCreateTextureFromFile( myD3DDevicePtr, textureFile, &myBumpMapPtr ) )

	if( !myBumpMapPtr )
	{
		return E_FAIL;
	}
	return S_OK;
}

HRESULT CBumpMappedMeshNode::loadFX()
{
	// ltezik a fjl? - megprbljuk megnyitni
	CHECK_WEXIT( fileExists(myFXFile) )
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	
	CHECK_WLOG( D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0,//D3DXSHADER_DEBUG, 
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							) )
	// effekt betltsi hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		return E_FAIL;
	}

	// kezelk lekrdezse
	myHTech       	= myEffectPtr->GetTechniqueByName("BumpMapTech");
	myHWVP        	= myEffectPtr->GetParameterByName(0, "gWorldViewProjection");
	myHWorld 	  	= myEffectPtr->GetParameterByName(0, "gWorld");
	myHWorldInv		= myEffectPtr->GetParameterByName(0, "gWorldInverse");
	
	myHAmbientMaterialColor	= myEffectPtr->GetParameterByName(0, "gAmbientMaterialColor");
	myHDiffuseMaterialColor = myEffectPtr->GetParameterByName(0, "gDiffuseMaterialColor");
	myHSpecularMaterialColor = myEffectPtr->GetParameterByName(0, "gSpecularMaterialColor");
	
	myHLightColor = myEffectPtr->GetParameterByName(0, "gLightColor");	
	myHLightPos = myEffectPtr->GetParameterByName(0, "gLightPos");
	myHCameraPos = myEffectPtr->GetParameterByName(0, "gCameraPos");
	
	myHTex0 		 = myEffectPtr->GetParameterByName(0, "gTex0" );
	myHBumpMap 		 = myEffectPtr->GetParameterByName(0, "gBumpMap" );
	
	myHSpecularPower = myEffectPtr->GetParameterByName(0, "gSpecularParam" );
	
	if( !myHTech
		|| !myHWVP
		|| !myHWorld
		|| !myHWorldInv
		|| !myHAmbientMaterialColor
		|| !myHDiffuseMaterialColor
		|| !myHSpecularMaterialColor
		|| !myHLightColor
		|| !myHLightPos
		|| !myHCameraPos
		|| !myHTex0
		|| !myHBumpMap
		|| !myHSpecularPower )
	{
		LOG_ERROR( "Effekt paramter beolvassi hiba!" );
		return E_FAIL;
	}

	myEffectPtr->SetFloat( myHSpecularPower, mySpecularPower );

	return S_OK;
}

const D3DXMATRIX& CBumpMappedMeshNode::getWorldMat() 
{		
		myWorldMat = myScaleMat * myRotMat * myTranslMat;
		return myWorldMat;
}

void CBumpMappedMeshNode::update( float dt )
{
	// lassan forgatjuk a modellt a sajt tengelye krl
	static const D3DXVECTOR3 rotationStep( 0.f, 0.1f, 0 );
	myRotVec += rotationStep * dt;

	D3DXMatrixRotationYawPitchRoll( &myRotMat, myRotVec.y, myRotVec.x, myRotVec.z );
}

void CBumpMappedMeshNode::render()
{
	myEffectPtr->SetTechnique(myHTech);

	myD3DDevicePtr->SetVertexDeclaration(myDeclPtr);

	myEffectPtr->SetMatrix(myHWVP, &( getWorldMat() * GameCore->Camera()->getViewProjMatrix() ));

	myEffectPtr->SetMatrix(myHWorld, &getWorldMat());

	D3DXMatrixTranspose( &myInvWorldMat, &myWorldMat );
	myEffectPtr->SetMatrix(myHWorldInv, &myInvWorldMat );
		
	myEffectPtr->SetValue(myHAmbientMaterialColor, &myAmbientMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHDiffuseMaterialColor, &myDiffuseMat, sizeof(D3DXCOLOR));
	myEffectPtr->SetValue(myHSpecularMaterialColor, &mySpecularMat, sizeof(D3DXCOLOR));
	
	myEffectPtr->SetValue(myHLightPos, &myLightPos, sizeof(D3DXVECTOR3));
	myEffectPtr->SetValue(myHLightColor, &myDiffuseLight, sizeof(D3DXCOLOR));
	
	myEffectPtr->SetValue( myHCameraPos, &( GameCore->Camera()->getPosition() ), sizeof(D3DXVECTOR3) );
	
	myEffectPtr->SetTexture( myHBumpMap, myBumpMapPtr );
	
	myMeshPtr->render( myEffectPtr, myHTex0, NULL /* no mat */, true /* draw opaque subsets */ );
}

void CBumpMappedMeshNode::onLostDevice()
{
	myEffectPtr->OnLostDevice();
}

void CBumpMappedMeshNode::onResetDevice()
{
	myEffectPtr->OnResetDevice();
}

}
}

