/* ----------------------------------------------
	Nyisztor Kroly
	Shader programozs
	Grafika s jtkfejleszts DirectX-szel II
	
	FX fjl	: EnvironmentMap.fx
	Lers	: Krnyezetlekpzs krnyezeti textrbl
	Lsd	: Grafika s jtkfejleszts DirectX-szel
			  11.9.4 Fnyvisszavero s fnytro krnyezetlekpzs
			  408. oldal
------------------------------------------------- */
uniform extern float4x4 gWorldViewProjection;
uniform extern float4x4 gWorld;
uniform extern float4x4 gWorldInverse;
uniform extern float4 	gAmbientMaterialColor;
uniform extern float4 	gDiffuseMaterialColor;
uniform extern float4 	gSpecularMaterialColor;
uniform extern float4 	gLightColor;
uniform extern float3 	gLightPos;
uniform extern float3   gCameraPos;
uniform extern texture  gTex0;
uniform extern texture  gEnvMap;
uniform extern float gSpecularParam;
// az anyag fnyvisszaver tulajdonsga
uniform extern float  gMatReflectivity;


sampler SamplerTex0 = sampler_state
{
	Texture = <gTex0>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler SamplerEnvironmentMap = sampler_state
{
	Texture = <gEnvMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};
 
struct VS_OUTPUT
{
    float4 PositionProj : POSITION0;
    float2 Tex0 	    : TEXCOORD0;
    float3 NormalW      : TEXCOORD1;
    float3 ObjCamDirW   : TEXCOORD2;
	float3 LightVecW    : TEXCOORD3;
};

VS_OUTPUT vs_main(float3 posLocal_in : POSITION0, 
				  float3 normalLocal_in : NORMAL0, 
				  float2 tex0_in: TEXCOORD0)
{
    // eredmny struktra kinullzsa
	VS_OUTPUT outVS = (VS_OUTPUT)0;

	// textra-koordintk tadsa a raszterizcis szakasznak
	outVS.Tex0 = tex0_in;
	
	// a cscsponti normlvektort vilg koordintarendszerbe transzformljuk
	float3 normalW = mul(float4(normalLocal_in, 0.0f), gWorld).xyz;
	outVS.NormalW = normalize(normalW);
	
	// a cscspont helyzett is vilg koordintarendszerbe transzformljuk
	float3 posW = mul(float4(posLocal_in, 1.0f), gWorld).xyz;
	// objektum-kamera irny kiszmtsa
	outVS.ObjCamDirW = normalize( gCameraPos - posW );
	// fnyvektor kiszmtsa
	outVS.LightVecW = normalize( gLightPos - posW );
	// cscspont helyzet transzformci vgrehajtsa
	outVS.PositionProj = mul(float4(posLocal_in, 1.0f), gWorldViewProjection);
	// a kitlttt struktra visszatrtse
    return outVS;
}
// kppont rnyal
float4 ps_main(float2 tex0_in : TEXCOORD0,
				float3 normalW_in : TEXCOORD1, 
                float3 objCamDirW_in : TEXCOORD2,
				float3 lightVecW_in  : TEXCOORD3) : COLOR
{
	// normalizljuk a raszterizcis szakaszban 
	// kiszmtott vektorokat, ugyanis 
	// az interpolci nem eredmnyez felttlenl 
	// egysgvektorokat
	float3 normalWorld = normalize(normalW_in);
	float3 objCamDirWorld = normalize(objCamDirW_in);
	float3 lightWorld = normalize(lightVecW_in);
	
	// fnyintenzits kiszmtsa a Lambert-fle koszinusz fggvnnyel
	float i_diffuse = max(dot(lightWorld, normalWorld), 0.0f);
	// a diffz szn kiszmtsa
	float3 diffuseColor = i_diffuse * ( gDiffuseMaterialColor * gLightColor).rgb;

	// a felletrl visszavert fny irnynak kiszmtsa
	float3 r = reflect(-lightWorld, normalWorld);
	
	// a fnyfolt intenzitsnak kiszmtsa
	float i_spec = pow(max(dot(r, objCamDirWorld),0.0f), gSpecularParam);
	// fnyfolt sznnek kiszmtsa
	float3 specColor = i_spec*( gSpecularMaterialColor * gLightColor).rgb;	
	
	// texel lekrdezse a modell textrjbl
	float4 texColor = tex2D(SamplerTex0, tex0_in);
	
	// a felletrl a nzpontba visszavert vektor
	// ezt hasznljuk mintavtelezshez a krnyezeti textrbl
	//float3 envMapSamplerRay = reflect(-objCamDirWorld, normalWorld);
	float3 envMapSamplerRay = refract(-objCamDirWorld, normalWorld, 0.95f);
	
	float3 environmentColor = texCUBE(SamplerEnvironmentMap, envMapSamplerRay);
	
	// slyozssal kiszmtjuk, mennyire rvnyesl a tkrzds a felsznen, s mennyire 
	// a modell sajt szne. Minl kisebb gMatReflectivity rtke, annl jobban ltszik a modell textrja.
	float3 reflectedColor = environmentColor * gMatReflectivity + (1.0f-gMatReflectivity) * (gAmbientMaterialColor + diffuseColor) * texColor;
	
	// kimeneti struktra feltltse
	return float4( reflectedColor + specColor, gDiffuseMaterialColor.a * texColor.a );
}

technique EnvironmentMapTech
{
    pass P0
    {
        vertexShader = compile vs_2_0 vs_main();
        pixelShader  = compile ps_2_0 ps_main();
    }
}