﻿/* Shaderprogramozás - Grafika és játékfejlesztés DirectX-szel ----------------

	Állomány          : CTimer.cpp

    Nyelv             : C++

    Létrehozás dátuma : 2008.08.28

    Utolsó módosítás  : 2009.03.15

    Leírás            : Shaderprogramozás
						Grafika és játékfejlesztés DirectX-szel
						11.1.8. "Időzítés"
						291. oldal

               Copyright (C) Nyisztor Károly 2008-2009
							www.directx.hu
-----------------------------------------------------------------------------*/
/*- Szükséges függvénykönyvtárak --------------------------------------------*/

/*- Befordított fejállományok -----------------------------------------------*/
#include "CTimer.h"

/*- Névterek ----------------------------------------------------------------*/
namespace app
{
namespace timer
{

/*- Definíciók és konstansok ------------------------------------------------*/
/*- Globális változók -------------------------------------------------------*/

/*- Implementációs rész -----------------------------------------------------*/
/*!
 * \brief
 * konstruktor
 * 
 * \remarks
 * bővebben: 292.-293. oldal
 */
CTimer::CTimer()
{
	init();
}
/*!
 * \brief
 * az időzítő inicializálása
 * 
 * \remarks
 * bővebben: 292.-293. oldal
 */
void CTimer::init()
{
	//idözíto frekvenciája - ennyiszer frissül egy másodperc alatt
	if ( QueryPerformanceFrequency( reinterpret_cast<LARGE_INTEGER *>( &myFreq ) ) )
	{ 		
		myPerfCounterAvailable = true;		
	}
	else
	{
		myPerfCounterAvailable = false;
		myFreq = 1000;
	}
	myTimeScale = 1.f / myFreq;
	// időzítő elindítása
	resetTimer();
}
/*!
 * \brief
 * elindítja, illetve újraindítja az időzítőt
 * (újra)inicializálja az adattagokat 
 */
void CTimer::resetTimer()
{	
	myLastElapsedTime = 0;	
	myElapsedTime = 0;
	myCurrentTime = 0;
	myLastTimeHere = 0;
	myElapsedTime = 0;

	myFramesPerSec = 0;
	myTotalFrameCnt = 0;
	
	myCurrentFPS = 0;
	myMinFPS = 0x7fffffff;
	myMaxFPS = 0;
	
	myMinFrameTime = 0.f;
	myMaxFrameTime = 0x7fffffff;
	myCurrentFrameTime = 0.f;
	
	//stopper indításának időpontja
	myStartTime = getTime();
	myUpdateStartTime = myStartTime;
}
/*!
 * \brief
 * időzítő adatok naplózása
 */
void CTimer::logTimer()
{	
	FILE* filePtr;

	DWORD fps = getFPS();

	fopen_s( &filePtr, "stats.log", "w" );
	if( filePtr )
	{
		fprintf( filePtr,"avg fps: %d\n", fps );
		fprintf( filePtr,"min fps: %d\n", myMinFPS );
		fprintf( filePtr,"max fps: %d\n", myMaxFPS );
		fprintf( filePtr,"min fdt: %0.4f\n", myMinFrameTime );
		fprintf( filePtr,"max fdt: %0.4f\n", myMaxFrameTime );

		fclose( filePtr );
	}
}
/*!
 * \brief
 * átlag fps lekérdezése
 * 
 * \returns
 * fps érték dupla lebegőpontos értékként
 */
const double CTimer::getAvgFPS()
{		
	__int64 t1 = getTime();
	__int64 dt = t1 - myStartTime;
	// átlag FPS a futás eddigi pillanatáig
	double fps = 0.f;
	if(dt != 0.f)
	{
		fps = ( static_cast<double>(myTotalFrameCnt) / dt ) * myFreq;
	}
	return fps;
}
/*!
 * \brief
 * átlag fps lekérdezése
 * 
 * \returns
 * fps érték szövegként
 */
const char* CTimer::getAvgFPSString()
{
	double fps = getAvgFPS();
	sprintf_s( myFPSStrBuffer, "AVG:%.3f", fps );
	return myFPSStrBuffer;
}
/*!
 * \brief
 * jelenlegi fps lekérdezése
 * 
 * \returns
 * fps érték DWORD értékként
 */
const DWORD CTimer::getFPS() const
{
   return myCurrentFPS;
}
/*!
 * \brief
 * jelenlegi fps lekérdezése
 * 
 * \returns
 * fps érték szövegként
 */
const char* CTimer::getFPSString()
{
	double fps = getFPS();
	sprintf_s( myFPSStrBuffer, "FPS:%.3f", fps );
	return myFPSStrBuffer;
}

/*!
 * \brief
 * képkockák közt eltelt idő
 * 
 * \returns
 * megjelenített képkockák között eltelt idő, dupla lebegőpontos érték
 */
const double CTimer::getFrameTime() const
{
	return myCurrentFrameTime;
}
/*!
 * \brief
 * képkockák közt eltelt idő
 * 
 * \returns
 * megjelenített képkockák között eltelt idő, szövegként
 */
const char* CTimer::getFrameTimeString() 
{
	double str = getFrameTime();
	sprintf_s( myFPSStrBuffer, "FDT:%.3f", str );
	return myFPSStrBuffer;
}
/*!
 * \brief
 * időzítő frissítése
 */
void CTimer::update()
{
	++myFramesPerSec;
	++myTotalFrameCnt;
	// eltelt egy másodperc?
	if( ( getTime() - myUpdateStartTime ) > myFreq )
	{
		// képkockák közti idő
		myCurrentFrameTime = static_cast<double>(1.0f / myFramesPerSec);
	
		// min és max FPS karbantartása
		if( myFramesPerSec < myMinFPS )
		{
			myMinFPS = myFramesPerSec;
			myMaxFrameTime = myCurrentFrameTime;
		}

		if( myFramesPerSec > myMaxFPS )
		{
			myMaxFPS = myFramesPerSec;
			myMinFrameTime = myCurrentFrameTime;
		}

		myCurrentFPS = myFramesPerSec;

		// újrainicializálás
		myFramesPerSec = 0;		
		myUpdateStartTime = getTime();
	}
};

/*!
 * \brief
 * alapvető időmérő függvény
 * 
 * \returns
 * a precíz időzítő számlálójának értéke
 *
 * \remarks
 * bővebben: 294. oldal
 */
_int64 CTimer::getTime()
{
	_int64 aTime = 0;
	if( myPerfCounterAvailable )
	{
		QueryPerformanceCounter( reinterpret_cast<LARGE_INTEGER *>( &aTime ) );
	}
	else
	{
		aTime = timeGetTime();
	}
	return aTime;
}
/*!
 * \brief
 * a függvény két, egymás utáni hívása között eltelt időt adja vissza
 * 
 * \returns
 * idő másodpercben
 *
 * \remarks
 * bővebben: 294. oldal
 */
double CTimer::getTimeElapsed()
{
	// elozo érték elmentése
	myLastElapsedTime = myElapsedTime;
	// jelenlegi ido lekérdezése
	myCurrentTime = getTime();	
  
	myElapsedTime = ( myCurrentTime - myLastTimeHere ) * myTimeScale;
	myLastTimeHere = myCurrentTime;

	const double aSmoothFactor = 10.0f;
	if ( myElapsedTime < ( myLastElapsedTime * aSmoothFactor ) )
	{
		return myElapsedTime;
	}
	else
	{
		return 0.0f;
	}

	return myElapsedTime;	
}

} // namespace timer
} // namespace game